unit MyRobot;

interface

uses
  SysUtils, Token;

type
  Point = Record
    x,y: Integer;
end;

type
  TMyRobot = Class (TObject)
  public
    constructor Create(Data: String);
    destructor Destroy(); override;
    procedure setLocationX(x: Integer);
    procedure setLocationY(y: Integer);
    procedure setDest(pEnd: Point; path: String);
    procedure setDelivery(pDest: Point; path, list: String);
    procedure dropDelivery();
    procedure onPathMove();

    procedure doMove(dir: Char);

    function getMove(): String;
    function atDest(): Boolean;
    function waitOnPackage(): Boolean;
  private
    dataReady: Boolean; // My Robot's Data is ready and valid!
    id: Integer;        // My Robot's Identifier
    cap: Integer;       // My Robot's Capacity
    money: Integer;     // My Robot's Money
    myLocation: Point;  // My Robot's Location

    // State Info
    myState: Integer;
    myHaveDest: Boolean;
    myDest: Point;
    myPath: String;
    myOnPath: Boolean;
    waitForPackage: Boolean;
    packageList: String;

    // Move Info
    myBid: Integer;     // Bid to Move
    myMove: String;     // Given Move
    myMoveData: String; // Move's Data
  published
    property identifier: Integer read id;
    property capacity: Integer read cap;
    property cash: Integer read money;
    property location: Point read myLocation;
    property ready: Boolean read dataReady;
    property state: Integer read myState;
    property haveDest: Boolean read myHaveDest write myHaveDest;
    property dest: Point read myDest;
    property path: String read myPath;
    property onPath: Boolean read myOnPath;
    property bid: Integer read myBid write myBid;
    property move: String read myMove write myMove;
    property moveData: String read myMoveData write myMoveData;
end;

const
  ROBOT_FINDPACKAGE = $00;
  ROBOT_DELIVER = $01;
  STATE_STR: array[0..1] of String =
        ('Finding Package',
         'Dilivering Package');
  STMOVE_NORTH = 'S';
  STMOVE_EAST = 'E';
  STMOVE_SOUTH = 'N';
  STMOVE_WEST = 'W';
  MOVE_NORTH = $01;
  MOVE_EAST = $02;
  MOVE_SOUTH = $03;
  MOVE_WEST = $04;


implementation

constructor TMyRobot.Create(Data: String);
var
  myToken: TToken;
begin
  inherited Create();

  myToken := TToken.Create(Data, ' ', '+');
  if (myToken.countTokens = 3) then begin
    id := StrToInt(myToken.nextToken());
    cap := StrToInt(myToken.nextToken());
    money := StrToInt(myToken.nextToken());
    dataReady := true;
    WriteLn('I''m playing as #' + IntToStr(id) + ', Capacity=' + IntToStr(cap) +
            ', and Money=' + IntToStr(money));
    myLocation.x := 0;
    myLocation.y := 0;
    myState := ROBOT_FINDPACKAGE;
    myHaveDest := false;
    myDest.x := 0;
    myDest.y := 0;
    myPath := '';
    myOnPath := false;
    waitForPackage := false;
  end
  else begin
    WriteLn('Malformed Player Data');
    dataReady := false;
  end;
end;

destructor TMyRobot.Destroy();
begin

  inherited Destroy();
end;

procedure TMyRobot.setLocationX(x: Integer);
begin
  myLocation.x := (x-1);
end;

procedure TMyRobot.setLocationY(y: Integer);
begin
  myLocation.y := (y-1);
end;

procedure TMyRobot.setDest(pEnd: Point; path: String);
begin
  myDest := pEnd;
  myPath := path;
  myHaveDest := true;
  myMove := 'Move';
  myMoveData := myPath[1];
  WriteLn('Moving to ' + IntToStr(pEnd.x) + ',' + IntToStr(pEnd.y) + ' - ' + path);
  waitForPackage := false;
end;

procedure TMyRobot.setDelivery(pDest: Point; path, list: String);
begin
  myDest := pDest;
  myPath := path;
  myHaveDest := false;
  myMove := 'Pick';
  myMoveData := list;
  WriteLn('Delivering to ' + IntToStr(pDest.x) + ',' + IntToStr(pDest.y) + ' - ' + path);
  waitForPackage := false;
  packageList := list;
  myState := ROBOT_DELIVER;
end;

procedure TMyRobot.dropDelivery();
begin
  myHaveDest := false;
  myMove := 'Drop';
  myMoveData := packageList;
  myState := ROBOT_FINDPACKAGE;
end;

procedure TMyRobot.onPathMove();
begin
  if (length(myPath) > 0) then begin
    myMove := 'Move';
    myMoveData := myPath[1];
  end;
end;

procedure TMyRobot.doMove(dir: Char);
begin
  case (dir) of
    STMOVE_NORTH:       dec(myLocation.y);
    STMOVE_SOUTH:       inc(myLocation.y);
    STMOVE_WEST:        dec(myLocation.x);
    STMOVE_EAST:        inc(myLocation.x);
  end;
  if (length(myPath) > 0) and (dir = myPath[1]) then begin
    myPath := Copy(myPath, 2, length(myPath));
    myOnPath := true;
  end
  else begin
    myOnPath := false
  end;
end;

function TMyRobot.getMove(): String;
begin
  myBid := 1;
  dec(Money, myBid);
  Result := IntToStr(myBid) + ' ' + MyMove + ' ' + myMoveData;
  if (myMove = 'Move') then
    WriteLn('Me (' + IntToStr(myLocation.x) + ',' + IntToStr(myLocation.y) +
            ',' + IntToStr(Money) + ') - ' + STATE_STR[myState] + '::' +
            Result + ' (' + myPath + ')')
  else
    WriteLn('Me (' + IntToStr(myLocation.x) + ',' + IntToStr(myLocation.y) +
            ',' + IntToStr(Money) + ') - ' + STATE_STR[myState] + '::' + Result);

end;

function TMyRobot.atDest(): Boolean;
begin
  Result := (myHaveDest) and (length(myPath) = 0);
end;

function TMyRobot.waitOnPackage(): Boolean;
begin
  Result := waitForPackage;
  waitForPackage := true;
end;

end.
