unit Network;

interface

uses
  SysUtils, IdGlobal, IdBaseComponent, IdComponent, IdTCPClient, Map;


type
  TRobotLoop = Class (TObject)
  public
    constructor Create();
  private
    myMap: TMap;
    tcpClient: TIdTCPClient;

    procedure tcpOnConnect(Sender: TObject);
end;

implementation

const
  STATE_NOMAP = $0001;
  STATE_MAPDATA = $0002;
  STATE_PLAYERCFG = $0003;
  STATE_GAMESTART = $0004;
  STATE_GAMELOOP = $0005;
  DEBUG = false;

constructor TRobotLoop.Create();
var
  host: String;         // Host
  port: Integer;         // Port
begin
  if (ParamStr(1) = '') then
    host := 'localhost'
  else
    host := ParamStr(1);
  if (ParamStr(2) = '') then
    port := 20005
  else
    port := StrToInt(ParamStr(2));
  WriteLn('Connecting to ' + host + ':' + IntToStr(port));
  tcpClient := TIdTCPClient.Create(nil);
  tcpClient.Host := host;
  tcpClient.Port := port;
  tcpClient.OnConnected := tcpOnConnect;
  tcpClient.MaxLineLength := 256*256;
  tcpClient.Connect();
end;

// We've connecting, send Player
procedure TRobotLoop.tcpOnConnect(Sender: TObject);
var
  myState: Integer;
  line: String;
  send: String;
  move: Boolean;
begin
  // Send Player
  tcpClient.WriteLn('Player');
  myState := STATE_NOMAP;
  move := true;

  // Read lines while we'll we're connected, be sure to close the socket
  // after we're done.

  while (tcpClient.Connected) do begin
    line := tcpClient.readLn();
    if (DEBUG) then
      WriteLn('Read: ' + line);
    case (myState) of
      STATE_NOMAP:      begin
                          myMap := TMap.Create(line);
                          myState := STATE_MAPDATA;
                        end;
      STATE_MAPDATA:    begin
                          myMap.ParseLine(line);
                          if (myMap.haveAllData) then
                            myState := STATE_PLAYERCFG;
                        end;
      STATE_PLAYERCFG:  begin
                          myMap.CreateMyPlayer(line);
                          myState := STATE_GAMESTART;
                        end;
      STATE_GAMESTART:  begin
                          myMap.ParseGameLine(line);
                          move := true;
                          myState := STATE_GAMELOOP;
                        end;
      STATE_GAMELOOP:   begin
                          // Check to see if this is player data or packages
                          move := (length(line) > 0) and (line[1] = '#');
                          if (move) then begin
                            myMap.ParseGameLine(line);
                            myMap.ParsePackageLine();
                          end
                          else begin
                            myMap.setPackageLine(line);
                          end;
                        end;
    end;
    // Calculate and send our move
    if (myState = STATE_GAMELOOP) and (move) then begin
      myMap.robotAI();
      send := myMap.me.getMove();
      if (DEBUG) then
        WriteLn('Send: ' + send);
      tcpClient.WriteLn(send + #$10);
      if (DEBUG) then
        Sleep(1000);
    end;
  end;

  // Game's Over, close the connection and clean up!
  tcpClient.Disconnect();
  tcpClient.Free();
  tcpClient := nil;
  myMap.Free();
  myMap := nil;
end;

end.
